
/**
 * A module defining `ModbusReadPointsConfig`.
 *
 * @module nmodule/honIrmControl/rc/ModbusReadPointsConfig
 */
define(['baja!',
        'bajaux/Widget',
        'css!nmodule/honIrmControl/rc/css/irmnx-styles',
        'css!nmodule/honIrmControl/rc/css/comp-table',
        'nmodule/honIrmControl/rc/ModbusUtil',
        'lex!honIrmControl',
        'dialogs',
        'jquery'], function ( 
        baja,
        Widget,
        css,
        tablecss,
        ModbusUtil,
        lexicons,
        dialogs,
        $ ) {
	
	
	  'use strict';	
	  var honIrmControlLex = lexicons[0];
	
	var ModbusReadPointsConfig = function ModbusReadPointsConfig() {
	      Widget.apply(this, arguments);
	  };
	  
	  ModbusReadPointsConfig.prototype = Object.create(Widget.prototype);
	  ModbusReadPointsConfig.prototype.constructor = ModbusReadPointsConfig;  
	  
	  var modbusUtil = new ModbusUtil();
	  
	  
	  ModbusReadPointsConfig.prototype.getReadPointTableId = function(dom) {		
			return dom.find('#readPointTableBodyId');
		}
	  
	    //read pts filter
      ModbusReadPointsConfig.readPtAddressFilterDsc = "readPtAddressFilterDsc";
	  ModbusReadPointsConfig.readPtAddressFilterAsc = "readPtAddressFilterAsc";
	  ModbusReadPointsConfig.readPtNameFilterDsc = "readPtNameFilterDsc";
	  ModbusReadPointsConfig.readPtNameFilterAsc = "readPtNameFilterAsc";
	  ModbusReadPointsConfig.activeFilterReadPts = ModbusReadPointsConfig.readPtAddressFilterAsc;
	  ModbusReadPointsConfig.readPtTypeFilterDsc = "readPtTypeFilterDsc";
	  ModbusReadPointsConfig.readPtTypeFilterAsc = "readPtTypeFilterAsc";
	  ModbusReadPointsConfig.readPtOperationModeFilterDsc = "readPtOperationModeFilterDsc";
	  ModbusReadPointsConfig.readPtOperationModeFilterAsc = "readPtOperationModeFilterAsc";
	
	  ModbusReadPointsConfig.selectAllReadPtIdChkBox = '#selecetAllReadPtIdChkBox';
	  ModbusReadPointsConfig.checkBoxReadPoint = '#checkBoxReadPoint';
	  ModbusReadPointsConfig.userDefinedMode = 5;

	  function sortReadTableByFilter(ptsArray,filter){
		  
		  if(filter === ModbusReadPointsConfig.readPtAddressFilterAsc){
			  ptsArray.sort(function(readPoint, readPointNext){
	    	    return readPoint.readPtAddress-readPointNext.readPtAddress;
	    	});
		  }else if(filter === ModbusReadPointsConfig.readPtAddressFilterDsc){
			  ptsArray.sort(function(readPoint, readPointNext){
		    	    return readPointNext.readPtAddress-readPoint.readPtAddress;
		    });  
		  }else if(filter === ModbusReadPointsConfig.readPtNameFilterDsc){
			  ptsArray.sort(function(readPoint, readPointNext){
		    	    return readPointNext.readPtName.localeCompare(readPoint.readPtName);
		   });
		  }else if(filter === ModbusReadPointsConfig.readPtNameFilterAsc){
			  ptsArray.sort(function(readPoint, readPointNext){
		    	    return readPoint.readPtName.localeCompare(readPointNext.readPtName);
		    });		  
	      }
		  else if(filter === ModbusReadPointsConfig.readPtTypeFilterDsc){
			  ptsArray.sort(function(readPoint, readPointNext){
		    	    return readPointNext.readOptionPointTypeVal.localeCompare(readPoint.readOptionPointTypeVal);
		    });
	      }else if(filter === ModbusReadPointsConfig.readPtTypeFilterAsc){
			  ptsArray.sort(function(readPoint, readPointNext){
		    	    return readPoint.readOptionPointTypeVal.localeCompare(readPointNext.readOptionPointTypeVal);
		    });
	      }else if(filter === ModbusReadPointsConfig.readPtOperationModeFilterDsc){
			  ptsArray.sort(function(readPoint, readPointNext){
		    	    return  readPointNext.readPtOperationModeVal.localeCompare(readPoint.readPtOperationModeVal);
		    });
	      }else if(filter === ModbusReadPointsConfig.readPtOperationModeFilterAsc){
			  ptsArray.sort(function(readPoint, readPointNext){
		    	    return readPoint.readPtOperationModeVal.localeCompare(readPointNext.readPtOperationModeVal);
		   });
	    }
	      
	      return ptsArray;
	  }
	  
	  
	  ModbusReadPointsConfig.prototype.sortReadTableOnSelectedFilter = function(widget,filter){
	      var dom = widget.jq();
	      
	      var readPtsArray =  sortReadTableByFilter(Object.values(widget.readPointDataList),filter);
		  	      
		  var readPointTableBody = this.getReadPointTableId(dom);
	      var readPointRows = readPointTableBody.find(".md-table-row");
		  var totalNoOfReadPts = readPointRows.length;
		  var objReadPointValues = Object.assign({},readPointRows);
		  
		  
		  
	      for (var i = 0; i <totalNoOfReadPts; i++) {   
	    	  var readPointRowIdHash = readPtsArray[i].rowIdJq;
	    	  var readPointRowId = readPointRowIdHash.substring(1,readPointRowIdHash.length);
		  	 objReadPointValues[i].setAttribute("id",readPointRowId);	  	   
		  	 var readPointcells = objReadPointValues[i].getElementsByClassName("modbus-comp-table-cell");
			  for (var l = 0; l <readPointcells.length; l++) { // 0 1 2 3
					 if( l==0 ){
						 readPointcells[l].innerText = readPtsArray[i].readPtName;
						 readPointcells[l].setAttribute("data",readPtsArray[i].readPtName);
						 readPointcells[l].setAttribute("title",readPtsArray[i].readPtName);
					 }
					 else if(l==1){		 
						 readPointcells[l].innerText = readPtsArray[i].readOptionPointTypeVal;
						 readPointcells[l].setAttribute("data",readPtsArray[i].readPtTypeOrdinal);
						 readPointcells[l].setAttribute("title",readPtsArray[i].readOptionPointTypeVal);
					 }			 
					 else if(l==2){
						 
						 if(readPtsArray[i].readPtAddress.toString().toUpperCase().includes("0X")){
							 readPointcells[l].innerText = modbusUtil.showAddressInDecAndHexFromHex(readPtsArray[i].readPtAddress);
						 }else{
							 readPointcells[l].innerText = modbusUtil.showAddressInDecAndHex(readPtsArray[i].readPtAddress);
						 }
						 
						 readPointcells[l].setAttribute("data", readPtsArray[i].readPtAddress);
					 }				 
					 else if(l==3){
						 readPointcells[l].innerText = readPtsArray[i].readPtOperationModeVal;
						 readPointcells[l].setAttribute("data",readPtsArray[i].readPtOperationMode); 
					 }
					 else if(l==4){
						 readPointcells[l].innerText = readPtsArray[i].readPtPollTime;
						 readPointcells[l].setAttribute("data",readPtsArray[i].readPtPollTime); 
					 }
					 else if(l==5){			 
						 readPointcells[l].innerText = readPtsArray[i].readPtScalingFactor;
						 readPointcells[l].setAttribute("data",readPtsArray[i].readPtScalingFactor); 
					 } 
					 else if(l==6){
						 readPointcells[l].innerText = readPtsArray[i].optionPointDataFormatVal;
						 readPointcells[l].setAttribute("data",readPtsArray[i].readPtDataFormat); 
					 } 
					 else if(l==7){
						 readPointcells[l].innerText = modbusUtil.showInHex(readPtsArray[i].readPtEventBitMasking);
						 readPointcells[l].setAttribute("data",readPtsArray[i].readPtEventBitMasking); 
					 }
					 else if(l==8){		 
						  var countStr= readPointRowId.substring(ModbusUtil.seventeenNumber),
						  count = Number(countStr);
						  var readPointTableCheckRowId = "checkBoxReadPoint"+count;						 
						  readPointcells[l].getElementsByTagName("input")[0].setAttribute("id",readPointTableCheckRowId);
					 }  
				  } 
		  
		  	  }				  
  }
	  	  
	  ModbusReadPointsConfig.prototype.checkAllCheckBoxSelectedFromReadTable = function(widget){
		  // If select all button is checked
	      var dom = widget.jq()
		  var readPointTableBody = this.getReadPointTableId(dom);
	      var readPointRows = readPointTableBody.find(".md-table-row");
		  var totalNoOfReadPts = readPointRows.length;
		  for (var i = 0; i <totalNoOfReadPts; i++) {
			  var readPointRowId = readPointRows[i].getAttribute("id");
			  var count= readPointRowId.substring(ModbusUtil.seventeenNumber),
			  count = Number(count);
			  var readPointTableCheckRowId = ModbusReadPointsConfig.checkBoxReadPoint+count;
			  if(dom.find(readPointTableCheckRowId).prop("checked") == false){
				  return false;
			  }
		  }	
		  return true;
		// selected check box row will be deleted
	  }
	  
	  ModbusReadPointsConfig.prototype.handleSelectAllCheckBoxStatusFromReadPointsTable= function(readPtRowChkBox,widget){
		  if($(readPtRowChkBox).prop("checked") == false){				  
			  $(ModbusReadPointsConfig.selectAllReadPtIdChkBox).prop("checked",false);			  
		     }else{
		    	 if(this.checkAllCheckBoxSelectedFromReadTable(widget) == true)
		    		 {
		    		  $(ModbusReadPointsConfig.selectAllReadPtIdChkBox).prop("checked",true);		
		    		 }
		     }		  
		  }
	  
	  ModbusReadPointsConfig.prototype.selectAllCheckBoxFromReadPointsTable = function(widget){
		  //If select all button is checked
		      var dom = widget.jq()
			  var readPointTableBody = this.getReadPointTableId(dom);
		      var readPointRows = readPointTableBody.find(".md-table-row");
			  var totalNoOfReadPts = readPointRows.length;
			  var objReadPointValues = Object.assign({},readPointRows);
			  for (var i = 0; i <totalNoOfReadPts; i++) {
				  var readPointRowId = objReadPointValues[i].getAttribute("id");
				  var count= readPointRowId.substring(ModbusUtil.seventeenNumber),
				  count = Number(count);
				  var readPointTableCheckRowId = ModbusReadPointsConfig.checkBoxReadPoint+count;
				  if(dom.find(ModbusReadPointsConfig.selectAllReadPtIdChkBox).prop("checked") == true){
					  dom.find(readPointTableCheckRowId).prop("checked",true);
				  }else{
					  dom.find(readPointTableCheckRowId).prop("checked",false);
				  }		  
			  }			  
			//selected check box row will be deleted  
		  }
	  
	  
	  ModbusReadPointsConfig.prototype.validateReadPtAddressAndType = function(modBusPtsReadList,modBusPt,modBusPtsWriteList){
		  for(var i in modBusPtsReadList){
			  //should not compare with itself
			  if(modBusPtsReadList[i].rowIdJq == modBusPt.rowIdJq){
				  continue;
			  }
			  if(modBusPtsReadList[i].readPtAddress == modBusPt.readPtAddress){		  
					  
				  if(modBusPtsReadList[i].readPtTypeOrdinal == modBusPt.readPtTypeOrdinal){
					  var isErrorShown = modbusUtil.showErrorDiaogOnAddEditPointOnAdress(true,modBusPtsReadList[i].readPtAddress,modBusPtsReadList[i].readPtName);
					  return isErrorShown;
				  }
			  }
			  if(modBusPtsReadList[i].readPtName === modBusPt.readPtName){
				  var isSameNameErrorShown = modbusUtil.showErrorDiaogOnAddEditPointSameName(true,modBusPt.readPtName);
				  return isSameNameErrorShown;
			  }
		  } 
		  
		  for(var j in modBusPtsWriteList){
			  if(modBusPtsWriteList[j].writePtName === modBusPt.readPtName){
				  var isSameNameErrorShownWritePt = modbusUtil.showErrorDiaogOnAddEditPointSameName(true,modBusPt.readPtName);
				  return isSameNameErrorShownWritePt;
			  } 
		  }
		  return false;	  
	 }
	  
	  ModbusReadPointsConfig.prototype.showErrorDiaogOnAddEditReadPoint = function(scalingFactorVal,readPointAddressVal,readPointNameVal,eventBitMaskingVal,readPtPollTime,readPtOperationModeVal){
		  
			 
		  var scalingFactorEmpty =  modbusUtil.checkStirngAsNullOrEmpty(scalingFactorVal);
		  var pointAddressrEmpty =  modbusUtil.checkStirngAsNullOrEmpty(readPointAddressVal);
		  var pointNameEmpty =  modbusUtil.checkStirngAsNullOrEmpty(readPointNameVal);
		  var eventBitMaskingEmpty =  modbusUtil.checkStirngAsNullOrEmpty(eventBitMaskingVal);
		  if(scalingFactorEmpty == true || pointAddressrEmpty == true || pointNameEmpty == true || eventBitMaskingEmpty == true){
			 dialogs.showOk({
				  title: honIrmControlLex.get('honIrmControlLex.readpoint.add.edit.error.title'),
				  content: honIrmControlLex.get('honIrmControlLex.readpoint.add.edit.error'),
				  private: false
			     })
				  .ok(function () {
				    return true;
				  });
			 return true;
		  }
		  if(scalingFactorVal >3 || scalingFactorVal <-3){
				 dialogs.showOk({
					  title: honIrmControlLex.get('honIrmControlLex.readpoint.add.edit.error.title'),
					  content: honIrmControlLex.get('honIrmControlLex.readpoint.add.edit.scalingFactor.error'),
					  private: false
				     })
					  .ok(function () {
					    return true;
					  });
					
				 return true;
		  }
		  if((readPtOperationModeVal == ModbusReadPointsConfig.userDefinedMode))
		 {
		    if( readPtPollTime >86400 ||  readPtPollTime <1){
				 dialogs.showOk({
					  title: honIrmControlLex.get('honIrmControlLex.readpoint.add.edit.error.title'),
					  content: honIrmControlLex.get('honIrmControlLex.readpoint.add.edit.readPtPollTime.error'),
					  private: false
				     })
					  .ok(function () {
					    return true;
					  });
					
				 return true;
		       }
			}
		  
		  //check for read point address nan value
		  if(isNaN(readPointAddressVal)){
				 dialogs.showOk({
					  title: honIrmControlLex.get('honIrmControlLex.readpoint.add.edit.error.title'),
					  content: honIrmControlLex.get('honIrmControlLex.readpoint.add.edit.readPointAddressVal.nan.error'),
					  private: false
				     })
					  .ok(function () {
					    return true;
					  });
					
				 return true;
		  }
		  //check for limit value
		  if(readPointAddressVal.toString().toUpperCase().includes("0X")){
			  if(readPointAddressVal > 0xFFFF || readPointAddressVal < 0) {
					 dialogs.showOk({
						  title: honIrmControlLex.get('honIrmControlLex.readpoint.add.edit.error.title'),
						  content: honIrmControlLex.get('honIrmControlLex.readpoint.add.edit.readPointAddressVal.hex.limit.error'),
						  private: false
					     })
						  .ok(function () {
						    return true;
						  });
						
					 return true;
				  
			  }
		  }else{
			  if(readPointAddressVal > 65535 || readPointAddressVal < 0 ){
					 dialogs.showOk({
						  title: honIrmControlLex.get('honIrmControlLex.readpoint.add.edit.error.title'),
						  content: honIrmControlLex.get('honIrmControlLex.readpoint.add.edit.readPointAddressVal.dec.limit.error'),
						  private: false
					     })
						  .ok(function () {
						    return true;
						  });
						
					 return true;
				  
			  }
		  }
		  
		  
		  if(isNaN(modbusUtil.showInHex(eventBitMaskingVal))){
				 dialogs.showOk({
					  title: honIrmControlLex.get('honIrmControlLex.readpoint.add.edit.error.title'),
					  content: honIrmControlLex.get('honIrmControlLex.readpoint.add.edit.eventBitMasking.nan.error'),
					  private: false
				     })
					  .ok(function () {
					    return true;
					  });
					
				 return true;
		  }
		  if(modbusUtil.showInHex(eventBitMaskingVal) >0xFFFFFFFF || modbusUtil.showInHex(eventBitMaskingVal) <0){
				 dialogs.showOk({
					  title: honIrmControlLex.get('honIrmControlLex.readpoint.add.edit.error.title'),
					  content: honIrmControlLex.get('honIrmControlLex.readpoint.add.edit.eventBitMasking.min.max.error'),
					  private: false
				     })
					  .ok(function () {
					    return true;
					  });
					
				 return true;
		  }
		  
	  }
	  ModbusReadPointsConfig.prototype.getModbusDeletedReadPointTableData = function(widget) {
			
	 		 var deleteReadPointList =  widget.deleteReadPointDataList;
	 		 var modbusDeviceDeleteInputContentsObj = baja.$('baja:Component');
	 		  
	 		 for (var key in deleteReadPointList){
	 			  var modbusDeviceDeletedReadPtArgsObj = baja.$('honIrmControl:ModbusReadPointDisplayArgs');
	 		      modbusDeviceDeletedReadPtArgsObj.setReadPointName( baja.SlotPath.escape(deleteReadPointList[key].readPtName));

	 			  modbusDeviceDeletedReadPtArgsObj.setOldFb(deleteReadPointList[key].oldFb);
	 			  modbusDeviceDeletedReadPtArgsObj.setOldPointName(deleteReadPointList[key].oldPointName);
	 			  modbusDeviceDeletedReadPtArgsObj.setExistingSlotPath(deleteReadPointList[key].existingSlotPath);
	 	          //while adding add the slot name with escaping the special character but while showing to user unescape and show to user
	 			  modbusDeviceDeleteInputContentsObj.add({
	 				slot: modbusDeviceDeletedReadPtArgsObj.getReadPointName(),
	 			    value: modbusDeviceDeletedReadPtArgsObj
	 			   })
	 			  .then(function (prop) {
	 				})
	 			  .catch(function (err) {
	 				   baja.error(err);				      
	 			  });
	 			}  	  
		  
		  return modbusDeviceDeleteInputContentsObj;
	  }
	  
	  
	  
	  ModbusReadPointsConfig.prototype.getModbusReadPointTableData = function(dom, widget) {

		  var readPointList =  widget.readPointDataList;
		  var modbusDeviceInputContentsObj = baja.$('baja:Component');
		  
		  var ptNameArr = [];
		  
		  for (var key in readPointList) {
			  var modbusDeviceReadPtArgsObj = baja.$('honIrmControl:ModbusReadPointDisplayArgs');
			  //while adding add the slot name with escaping the special character but while showing to user unescape and show to user
			  modbusDeviceReadPtArgsObj.setReadPointName( baja.SlotPath.escape(readPointList[key].readPtName));
			  ptNameArr.push(readPointList[key].readPtName);
			  var readPointTypeEnum = baja.DynamicEnum.make(parseInt(readPointList[key].readPtTypeOrdinal));
			  modbusDeviceReadPtArgsObj.setReadPointType(readPointTypeEnum);
			  if(readPointList[key].readPtAddress.toString().toUpperCase().includes("0X")){
				  modbusDeviceReadPtArgsObj.setReadPointAddress(parseInt(readPointList[key].readPtAddress.toString(),16));
			  }else{
				  modbusDeviceReadPtArgsObj.setReadPointAddress(readPointList[key].readPtAddress);  
			  }
			  
			  var readPointOperationModeTypeEnum = baja.DynamicEnum.make(parseInt(readPointList[key].readPtOperationMode));
			  modbusDeviceReadPtArgsObj.setOperationMode(readPointOperationModeTypeEnum);
			  modbusDeviceReadPtArgsObj.setPollTime(readPointList[key].readPtPollTime);
			  modbusDeviceReadPtArgsObj.setScalingFactor(readPointList[key].readPtScalingFactor);
			  var dataFormatTypeEnum = baja.DynamicEnum.make(parseInt(readPointList[key].readPtDataFormat));
			  modbusDeviceReadPtArgsObj.setDataFormat(dataFormatTypeEnum);
			  modbusDeviceReadPtArgsObj.setOldFb(readPointList[key].oldFb);
			  modbusDeviceReadPtArgsObj.setOldPointName(readPointList[key].oldPointName);
			  modbusDeviceReadPtArgsObj.setExistingSlotPath(readPointList[key].existingSlotPath);
			  var hexValue = modbusUtil.showInHex(readPointList[key].readPtEventBitMasking);
			  modbusDeviceReadPtArgsObj.setEventBitMasking(parseInt(hexValue)); //while saving send as decimal i.e. as long it will be saved
	          //while adding add the slot name with escaping the special character but while showing to user unescape and show to user
			  modbusDeviceInputContentsObj.add({
				slot: modbusDeviceReadPtArgsObj.getReadPointName(),
			    value: modbusDeviceReadPtArgsObj
			   })
			  .then(function (prop) {
				})
			  .catch(function (err) {
				   baja.error(err);				      
			  });

			}  	  
	 		for (var i = 0; i < ptNameArr.length; i++) { 
		   		   for(var j = i+1; j < ptNameArr.length; j++){
		   			   if(ptNameArr[i] === ptNameArr[j]){
		   				var isErrorShown = modbusUtil.showErrorDiaogOnAddEditPointSameName(true,ptNameArr[i]);
		   				if(isErrorShown == true){
		   					return true;
		   				}
		   			  }
		   		   }
		   		} 
		  
		  return modbusDeviceInputContentsObj;
	  }
	
	
	  return ModbusReadPointsConfig;
	
});
